from typing import List

from .StructuralDampingByFrequencyComponent import StructuralDampingByFrequencyComponent

StructuralDampingByFrequencyComponentArray = List[StructuralDampingByFrequencyComponent]
